from .elasticache_describe_cache_clusters_request import ElasticacheDescribeCacheClustersRequest as ElasticacheDescribeCacheClustersRequest
from .elasticache_describe_cache_clusters_response import ElasticacheDescribeCacheClustersResponse as ElasticacheDescribeCacheClustersResponse
from .elasticache_describe_replication_groups_request import ElasticacheDescribeReplicationGroupsRequest as ElasticacheDescribeReplicationGroupsRequest
from .elasticache_describe_replication_groups_response import ElasticacheDescribeReplicationGroupsResponse as ElasticacheDescribeReplicationGroupsResponse